﻿import com.delfiweb.forms.*

/**
 *	Classe abstraite, elle ne sera jamais instancié mais fournir un cadre
 * 	a n'importe quel étape de formulaire. Il faut absolument appeler le super
 * 	constructeur dans le constructeur de la classe fille afin de faire l'enregistrement
 * 	des variables a transmettre au formulaire.
 * 
 *	@author		Cedric Néhémie
 *	@version	0.1
 *	@see		MovieClip	
 *	@see		_Form	
 *	@class		Form
 */
class com.delfiweb.forms.Form extends MovieClip implements _Form
{
	public var nextStep:Number;				// l'index de l'étape suivante
	public var previousStep:Number;			// l'index de l'étape précédente
	public var parentStruct:FormStruct;		// référence au parent
	public var onChange:Function;			// évènement onChange
	public var resultObject:Object;			// l'objet contenant les variables du formulaire
	public var concernedVar:Array;			// tableau des variables concernées



	/**
	 *	Super constructeur des clips de formulaires, on l'appelle depuis le constructeur
	 * 	de la classe fille afin d'enregistrer quelles variables devront ensuite être
	 * 	transmise dans le résultat du formulaire.
	 * 
	 * 	@method	Form
	 * 	entrée :	concerned	Array	Tableau des variables concernée par le formulaire
	 *	utilisation :	super(["var1","var2"])
	 *	sortie :	Void
	 */
	private function Form (concerned:Array)		
	{
		resultObject = {};
		concernedVar = concerned;
	}	
	
	
	
	/**
	 *	Fonction a appelée en interne par un bouton ou autre et qui provoque
	 * 	l'envoi des informations au formulaire.
	 * 
	 * 	@method	next
	 *	utilisation :	myForm.next()
	 *	sortie :	Void
	 */
	public function next()
	{
		resultObject = {};
		// si il y a une étape suiavnte
		if(nextStep)
		{
			// on recopie dans un objet toutes les variables concernées
			for(var i in  this)
			{
				//trace(i)
				if(isVarConcerned(i))
				{
					resultObject[i] = this[i]
				}
			}
			// on informe la structure de formulaires des changements effectuées
			parentStruct.addFormResult(resultObject)
			parentStruct.next()
		}
	}
	
	
	
	/**
	 *	Fonction a appelée en interne par un bouton ou autre et qui provoque
	 * 	un retour dans le formulaire.
	 * 
	 * 	@method previous
	 *	utilisation :	myForm.previous();
	 *	sortie :	Void
	 */
	public function previous()
	{
		parentStruct.previous()
	}



	/**
	 *	Déclenche la fin d'un formulaire, on agit comme dans le cas d'un <i>next</i>,
	 * 	mais au lieu de rappeler une étape, on lance directement la fin du formulaire
	 * 	(<i>submit</i> au niveau de la structure).
	 * 
	 * 	@method	submit
	 *	utilisation :	myForm.submit();
	 *	sortie :	Void
	 */
	public function submit()
	{
		resultObject = {};
		// on copie toutes les variables concernées dans l'objet
		for(var i in  this)
		{
			if(isVarConcerned(i))
			{
				resultObject[i] = this[i]
			}
		}
		// puis on déclenche la fin du formulaire
		parentStruct.addFormResult(resultObject)
		parentStruct.submit()
	}
	
	
	
	/**
	 *	Déclenche la réinitialisation de l'étape en composant avec la
	 * 	fonction de la classe parente (<i>FormStruct</i>)
	 * 
	 * 	@method	reset
	 *	utilisation :	myForm.reset()
	 *	sortie :	Void
	 */
	public function reset()
	{
		parentStruct.reset()
	}
	
	
	
	/**
	 *	Cette fonction devra être écrasée dans les classes filles, et seulement
	 * 	si on a des actions a faire lors de la mort d'un formulaire (contrôle clavier, 
	 * 	autre enregistrement comme écouteur).
	 * 
	 * 	@method	remove
	 *	utilisation :	myForm.remove()
	 *	sortie :	Void
	 */
	public function remove()
	{}
	
	
	
	/**
	 *	Fonction interne dont le rôle est de vérifier qu'une variable
	 * 	de la classe doit bien être copiée dans le résultat du formulaire
	 * 
	 * 	@method	isVarConcerned
	 * 	entrée :	varName	String Le nom de la variable a vérfiée
	 *	utilisation :	if(isVarConcerned("var1")) { ... }
	 *	sortie :	Boolean	Oui|Non est-elle concernée ?
	 */
	private function isVarConcerned (varName:String):Boolean
	{
		// on concate tout le tableau des variables concernées
		var s = ","+concernedVar+","
		// si le nom de la variable est présent dans la chaîne alors on renvoi true
		if(s.indexOf(","+varName+",") != -1) return true;
		
		return false;
	}
}