﻿import com.delfiweb.forms.*

/**
 *	Cette classe régit les enchainements d'étape dans un formulaire, pour
 * 	ce faire on définit des suites d'étapes (voir <i>FormStep</i>).
 * 	Ensuite la classe se charge de faire se succéder les étapes tout en
 * 	récupérant les infos a chaque formulaire.
 * 
 *	@author		Cedric Néhémie
 *	@version	0.1
 *	@class		FormStruct
 */
class com.delfiweb.forms.FormStruct 
{
	private var etapes:Array;			// les étapes.
	private var formResults:Object;		// l'objet résultat du formulaire contenant toutes les infos
	private var etapeActu:MovieClip;	// l'étapes courante dans laquelle on se situe
	private var currentId:Number		// l'id de l'etape courante
	private var clip:MovieClip;			// le clip ou créer un formulaire
	
	public var onSubmit:Function;		// la fonction appelée lors de la validation
	public var onSubmitVerify:Function	// la fonction de vérification
	
	
	
	/**
	 *	On construit la structure en lui transmettant le tableau des étapes, 
	 *  et le clip dans lequel il doit les créer.
	 * 
	 * 	@method	FormStruct
	 * 	entrée :	st	Array	Les étapes (constitués de <i>FormStep</i>).
	 * 	entrée :	c	MovieClip	Le clip dans lequel créer les étapes.
	 *	utilisation :	var struct = new FormStruct([s1,s2], _root);
	 *	sortie :	Void
	 */
	public function FormStruct (st:Array, c:MovieClip)
	{
		etapes = st;
		clip = c;
		currentId = 0;
		
		// on initialise l'objet de résultat
		formResults = {};
	}
	
	
	
	/**
	 *	Permet de construire l'étape courante du formulaire, elle peut servir
	 * 	a lancer la séquence ou encore a réinitialiser une étape.
	 * 
	 * 	@method	buildForm
	 *	utilisation :	struct.buildForm();
	 *	sortie :	Void
	 */
	public function buildForm ()
	{
		// on récupére l'étape courante
		var step = etapes[currentId]
		
		// on prépare un objet initialisateur
		var initObj= {};
		initObj.nextStep = step.nextId;
		initObj.previousStep = step.previousId;
		initObj.parentStruct = this;
		// on créer le clip étape 
		etapeActu = clip.attachMovie(step.linkageId,"form",0,initObj);
	}
	
	
	
	/**
	 *	Lors du passage d'une étape a une autre les informations saisies sont alors
	 * 	ajoutés a l'emsemble des infos du formulaires.
	 * 
	 * 	@method	addFormResult
	 * 	entrée :	obj	Object	Objet contenant les variables de formulaire a insérer
	 *	utilisation :	struct.addFormResult (o);
	 *	sortie :	Void
	 */
	public function addFormResult (obj)
	{
		for(var i in obj)
		{
			formResults[i] = obj[i];
			//trace(i+" : "+obj[i])
		}
	}
	
	
	
	/**
	 *	On appelle l'étape suivant celle courante, il ne s'agit pas forcément de celle
	 * 	qui vient juste après dans le tableau. on peut aussi imaginer que l'étape suivante
	 * 	varie selon une saisie utilisateur.
	 * 
	 * 	@method	next
	 *	utilisation :	struct.next();
	 *	sortie :	Void
	 */
	public function next ()
	{
		currentId = etapeActu.nextStep;
		buildForm ()
	}
	
	
	
	/**
	 *	On appelle l'étape précédente, fonctionne comme next a ceci près que
	 * 	les infos de l'étape du formulaire courant ne sont pas recopié dans
	 * 	l'objet résultat.
	 * 
	 * 	@method	previous
	 *	utilisation :	struct.previous();
	 *	sortie :	Void
	 */
	public function previous ()
	{
		currentId = etapeActu.previousStep;
		buildForm ()
	}
	
	
	
	/**
	 *	Appelle la fin du formualaire, on pourra envisager d'utiliser
	 * 	une fonction de vérification lors de cette étape, du type :
	 * 	if(verif()) ...
	 * 	Dans cette fonction on réinitialise tout le formulaire a l'exception
	 * 	de cet objet résultat.
	 * 
	 * 	@method submit
	 *	utilisation :	struct.submit();
	 *	sortie :	Void
	 */
	public function submit ()
	{
		// si il y a une fonction de vérification
		if(onSubmitVerify)
		{
			// et qu'elle valide le formulaire
			if(onSubmitVerify(getResult()))
			{
				// alors on fait tt ce qu'on a faire
				currentId = 0;
				var obj = getResult ();
				
				/*for(var i in obj)
				{
					trace(i+" : "+obj[i])
				}*/
				// on supprime le clip
				clip.form.removeMovieClip()
				
				onSubmit(true, obj)
			}
			// si pas validé on ne fait rien.
			else
			{
				onSubmit(false)
			}
		}
		// si pas de fonction de vérification
		else
		{
			// on fait la soumission de façon classique
			currentId = 0;
			var obj = getResult ();
			
			/*for(var i in obj)
			{
				trace(i+" : "+obj[i])
			}*/
			// on supprime le clip
			clip.form.removeMovieClip()
			
			onSubmit(true, obj)
		}		
	}
	
	
	
	/**
	 *	Le reset d'un formulaire consiste exclusivement a recréer le
	 * 	formulaire en lieu et place de l'actuel, cela a pour effet de
	 * 	vider les infos du formulaire pour les remplacer par les valeurs
	 * 	par défaut (a avoir définit dans la classe)
	 * 
	 * 	@method	reset
	 *	utilisation :	struct.reset();	
	 *	sortie :	Void
	 */
	public function reset ()
	{
		buildForm ()
	}
	
	
	
	/**
	 *	Permet de basculer directement a une étape précise, pour
	 * 	recommencer la totalité du formulaire par exemple, ou pour
	 * 	par exemple basculer la cible du suivant dans une étape.
	 * 
	 * 	@method	goto
	 * 	entrée :	step	Number	L'id de l'étape
	 *	utilisation :	struct.goto(0);
	 *	sortie :	Boolean	L'étape éxiste t-elle ? 
	 */
	public function goto(step:Number):Boolean
	{
		//trace("in goto : "+step)
		if(step < etapes.length)
		{
			currentId = step;
			buildForm()
			return true;
		}
		else
		{
			return false;
		}
	}
	
	
	
	/**
	 *	Permet de récupérer a tous moment les infos que possède le
	 * 	formulaire. Elle renvoi les infos sous la forme d'un objet.
	 * 
	 * 	@method	getResult
	 *	utilisation :	var result = struct.getResult();
	 *	sortie :	Object	L'objet résultat du formulaire.
	 */
	public function getResult ():Object
	{
		return formResults;
	}
}